@@ -26,6 +26,18 @@
 #include "md-int.h"
 
 
+#ifdef LCF_REISERFS
+#ifndef REISERFS_SUPER_MAGIC
+#define REISERFS_SUPER_MAGIC 0x52654973
+#endif
+#ifndef REISERFS_SUPER_MAGIC_STRING
+#define REISERFS_SUPER_MAGIC_STRING "ReIsErFs"
+#endif
+#ifndef REISERFS_IOC_UNPACK
+#define REISERFS_IOC_UNPACK		_IOW(0xCD,1,long)
+#endif
+#endif
+
 #ifndef HDIO_GETGEO
 #define HDIO_GETGEO HDIO_REQ
 #endif
@@ -477,9 +489,11 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 	  device,BIOS_MAX_SECS,geo->sectors);
     if (!lba32 &&
       (geo->start+geo->sectors-1)/geo->heads/geo->sectors >= BIOS_MAX_CYLS &&
-      !nowarn)
-	fprintf(stderr,"Warning: device 0x%04x exceeds %d cylinder limit\n",
+      !nowarn) {
+	fprintf(stderr,"Warning: device 0x%04x exceeds %d cylinder limit.\n"
+        "   Use of the 'lba32' option may help on newer (EDD BIOS) systems.\n",
 	  device,BIOS_MAX_CYLS);
+    }
     if (verbose > 2) {
 	printf("Device 0x%04x: BIOS drive 0x%02x, %d heads, %d cylinders,\n",
 	  device,geo->device,geo->heads,geo->cylinders == -1 ? BIOS_MAX_CYLS :
@@ -572,8 +586,20 @@ int geo_comp_addr(GEOMETRY *geo,int offset,SECTOR_ADDR *addr)
 	die("Internal error: sector > 0 after geo_open_boot");
     block = offset/geo->spb/SECTOR_SIZE;
     if (geo->file) {
+#ifdef LCF_REISERFS
+	    struct statfs buf;
+
+	    fstatfs(geo->fd, &buf);
+	    if (buf.f_type == REISERFS_SUPER_MAGIC) {
+		if (ioctl (geo->fd, REISERFS_IOC_UNPACK, 1) == ENOSPC)
+			die("Cannot unpack ReiserFS file");
+		if (verbose > 3) printf("fd %d: REISERFS_IOC_UNPACK\n", geo->fd);
+	    }
+#endif
 	if (ioctl(geo->fd,FIBMAP,&block) < 0) pdie("ioctl FIBMAP");
-	if (!block) return 0;
+	if (!block) {
+	    return 0;
+	}
     }
     sector = block*geo->spb+((offset/SECTOR_SIZE) % geo->spb);
     sector += geo->start;
