@@ -1,6 +1,13 @@
 /* lilo.c  -  LILO command-line parameter processing */
+/*
+Copyright 1992-1998 Werner Almesberger.
+Copyright 1999-2000 John Coffman.
+All rights reserved.
 
-/* Copyright 1992-1998 Werner Almesberger. See file COPYING for details. */
+Licensed under the terms contained in the file 'COPYING' in the 
+source directory.
+
+*/
 
 
 #include <stdlib.h>
@@ -24,10 +31,9 @@
 #include "cfg.h"
 #include "identify.h"
 #include "partition.h"
+#include "probe.h"
 #include "md-int.h"
 
-#define S2(x) #x
-#define S(x) S2(x)
 #define MAX_TOKEN 200
 
 static void show_other(int fd)
@@ -230,7 +236,7 @@ static void usage(char *name)
       "[ -v ... ]\n",name);
     fprintf(stderr,"%7s%s [ -C config_file ] [ -b boot_device ] [ -c ] "
       "[ -l | -L ]\n","",name);
-    fprintf(stderr,"%12s[ -i boot_sector ] [ -m map_file ] [ -d delay ]\n","");
+    fprintf(stderr,"%12s[ -i boot_loader ] [ -m map_file ] [ -d delay ]\n","");
     fprintf(stderr,"%12s[ -v ... ] [ -t ] [ -s save_file | -S save_file ]\n",
       "");
     fprintf(stderr,"%12s[ -P fix | -P ignore ] [ -r root_dir ] [ -w ]\n","");
@@ -239,6 +245,7 @@ static void usage(char *name)
     fprintf(stderr,"%7s%s [ -C config_file ] -I name [ options ]\n","",name);
     fprintf(stderr,"%7s%s [ -C config_file ] [ -s save_file ] "
       "-u | -U [ boot_device ]\n","",name);
+    fprintf(stderr,"%7s%s -T help\t\t(to see a list of options)\n", "", name);
     fprintf(stderr,"%7s%s -V\n\n","",name);
     exit(1);
 }
@@ -247,7 +254,7 @@ static void usage(char *name)
 int main(int argc,char **argv)
 {
     char *name,*config_file,*reboot_arg,*identify,*ident_opt,*new_root;
-    char *uninst_dev;
+    char *tell_param, *uninst_dev, *param, ch;
     int query,more,version,uninstall,validate,do_md_install,pass;
     BOOT_SECTOR dummy;
     IMAGE_DESCR dummy2;
@@ -261,38 +268,55 @@ int main(int argc,char **argv)
     DT_ENTRY *disk;
 
     config_file = DFL_CONFIG;
-    reboot_arg = identify = ident_opt = new_root = uninst_dev = NULL;
+    tell_param = reboot_arg = identify = ident_opt = new_root = uninst_dev = NULL;
     pass = do_md_install = query = version = uninstall = validate = 0;
-    name = *argv++;
+    name = *argv;
     argc--;
     cfg_init(cf_options);
-    while (argc && **argv == '-') {
+    while (argc && **++argv == '-') {
 	argc--;
-	if (argv[0][2]) usage(name);
-	switch ((*argv++)[1]) {
+      /* first those options with a mandatory parameter */
+      /* Notably absent are "Rv" */
+	if (strchr("bCdDfiImPrsSuTU", ch=(*argv)[1])) {
+	    if ((*argv)[2]) param = (*argv)+2;
+	    else {
+		param = *++argv;
+		if(argc-- <= 0) usage(name);
+	    }
+	} else { 
+	    param = NULL;
+	}
+#if 0
+fprintf(stderr,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
+#endif
+	switch (ch) {
 	    case 'b':
-		if (!argc) usage(name);
-		cfg_set(cf_options,"boot",*argv++,NULL);
-		argc--;
+		cfg_set(cf_options,"boot",param,NULL);
 		break;
 	    case 'c':
 		cfg_set(cf_options,"compact",NULL,NULL);
 		compact = 1;
 		break;
+	    case 'C':
+		config_file = param;
+		break;
 	    case 'd':
-		if (!argc) usage(name);
-		cfg_set(cf_options,"delay",*argv++,NULL);
-		argc--;
+		cfg_set(cf_options,"delay",param,NULL);
 		break;
 	    case 'D':
-		if (!argc) usage(name);
-		cfg_set(cf_options,"default",*argv++,NULL);
-		argc--;
+		cfg_set(cf_options,"default",param,NULL);
 		break;
 	    case 'f':
-		if (!argc) usage(name);
-		cfg_set(cf_options,"disktab",*argv++,NULL);
-		argc--;
+		cfg_set(cf_options,"disktab",param,NULL);
+		break;
+	    case 'i':
+		cfg_set(cf_options,"install",param,NULL);
+		break;
+	    case 'I':
+		identify = param;
+		if (argc--) {
+		    ident_opt = *argv++;
+		}
 		break;
 	    case 'l':
 		cfg_set(cf_options,"linear",NULL,NULL);
@@ -303,23 +327,85 @@ int main(int argc,char **argv)
 		lba32 = 1;
 		break;
 	    case 'm':
+		cfg_set(cf_options,"map",param,NULL);
+		break;
+	    case 'P':
+		if (!strcmp(param,"fix"))
+		    cfg_set(cf_options,"fix-table",NULL,NULL);
+		else if (!strcmp(param,"ignore"))
+			cfg_set(cf_options,"ignore-table",NULL,NULL);
+		else usage(name);
+		break;
+	    case 'q':
+		query = 1;
+		break;
+	    case 'r':
 		if (!argc) usage(name);
-		cfg_set(cf_options,"map",*argv++,NULL);
+		new_root = param;
 		argc--;
 		break;
-	    case 'i':
+	    case 'R':
+	        if (*(param = (*argv)+2)) argc++;
+	        else if (argc) param = *++argv;
+	        else reboot_arg = "";
+	        
+		while (argc) {
+			if (!reboot_arg)
+			    *(reboot_arg = alloc(strlen(param)+1)) = 0;
+			else {
+			    param = *++argv;
+			    strcat(reboot_arg = ralloc(reboot_arg,
+			        strlen(reboot_arg)+strlen(param)+2)," ");
+			}
+			strcat(reboot_arg, param);
+			argc--;
+		    }
+#if 0
+fprintf(stderr,"REBOOT=\"%s\"\n", reboot_arg);		    
+#endif
+		break;
+	    case 's':
 		if (!argc) usage(name);
-		cfg_set(cf_options,"install",*argv++,NULL);
+		cfg_set(cf_options,"backup",param,NULL);
 		argc--;
 		break;
-	    case 'I':
-		if (!argc) usage(name);
-		identify = *argv++;
-		if (--argc) {
-		    ident_opt = *argv++;
-		    argc--;
+	    case 'S':
+		cfg_set(cf_options,"force-backup",param,NULL);
+		break;
+	    case 't':
+		test = 1;
+		break;
+	    case 'T':
+	        tell_param = param;
+	    	break;
+	    case 'u':
+		validate = 1;
+		/* fall through */
+	    case 'U':
+		uninstall = 1;
+		if (param) {
+		    if (argc) usage(name);
+		    uninst_dev = param;
 		}
 		break;
+	    case 'v':
+	        if ((*argv)[2]) param = (*argv)+2;
+	        else if (argc && argv[1][0]>='0' && argv[1][0]<='9') {
+	            param = *++argv;
+	            argc--;
+	        }
+	        if (param) 
+		    verbose = *param - '0';
+		else
+	            verbose++;
+		break;
+	    case 'V':
+		version = 1;
+		break;
+	    case 'w':
+		cfg_set(cf_options,"nowarn",NULL,NULL);
+		nowarn = 1;
+		break;
 	    case 'X':
 		printf("-DIMAGES=%d -DCODE_START_1=%d -DCODE_START_2=%d "
 		  "-DDESCR_SIZE=%d -DDSC_OFF=%d -DDSC_OFF2=%d -DDFCMD_OFF=%d "
@@ -332,78 +418,6 @@ int main(int argc,char **argv)
 		  (void *) &dummy.par_1.msg_len-(void *) &dummy,
 		  (void *) &dummy2.flags-(void *) &dummy2);
 		exit(0);
-	    case 'C':
-		if (!argc) usage(name);
-		config_file = *argv++;
-		argc--;
-		break;
-	    case 'S':
-		if (!argc) usage(name);
-		cfg_set(cf_options,"force-backup",*argv++,NULL);
-		argc--;
-		break;
-	    case 's':
-		if (!argc) usage(name);
-		cfg_set(cf_options,"backup",*argv++,NULL);
-		argc--;
-		break;
-	    case 'P':
-		if (!argc) usage(name);
-		if (!strcmp(*argv,"fix"))
-		    cfg_set(cf_options,"fix-table",NULL,NULL);
-		else if (!strcmp(*argv,"ignore"))
-			cfg_set(cf_options,"ignore-table",NULL,NULL);
-		    else usage(name);
-		argv++;
-		argc--;
-		break;
-	    case 'q':
-		query = 1;
-		break;
-	    case 'r':
-		if (!argc) usage(name);
-		new_root = *argv++;
-		argc--;
-		break;
-	    case 'R':
-		if (!argc) reboot_arg = "";
-		else while (argc) {
-			if (!reboot_arg)
-			    *(reboot_arg = alloc(strlen(*argv)+1)) = 0;
-			else strcat(reboot_arg = ralloc(reboot_arg,
-			      strlen(reboot_arg)+strlen(*argv)+2)," ");
-			strcat(reboot_arg,*argv++);
-			argc--;
-		    }
-		break;
-	    case 't':
-		test = 1;
-		break;
-	    case 'u':
-		validate = 1;
-		/* fall through */
-	    case 'U':
-		uninstall = 1;
-		if (argc) {
-		    if (argc-- > 1) usage(name);
-		    uninst_dev = *argv;
-		}
-		break;
-	    case 'v':
-	        if (argc && **argv>='0' && **argv<='9') {
-		    verbose = **argv - '0';
-		    argc--;
-		    argv++;
-		} else
-	            verbose++;
-		break;
-	    case 'V':
-		version = 1;
-		break;
-	    case 'w':
-		cfg_set(cf_options,"nowarn",NULL,NULL);
-		nowarn = 1;
-		break;
 	    default:
 		usage(name);
 	}
@@ -422,22 +436,33 @@ int main(int argc,char **argv)
 	    return 0;
 	}
 	printf(", Copyright (C) 1992-1998 Werner Almesberger\n"
-	       "'lba32' extensions Copyright (C) 1999,2000 John Coffman\n");
-        if (verbose>=4) printf("Released %s and compiled at %s on %s.\n",
+	       "Linux Real Mode Interface library Copyright (C) 1998 Josh Vanderhoof\n"
+	       "Development beyond version 21 Copyright (C) 1999-2000 John Coffman\n"
+	       );
+        if (verbose>0) printf("Released %s and compiled at %s on %s.\n",
 	        VERSION_DATE, __TIME__, __DATE__);
+#if 0	        
+	if (verbose>4) printf("Secondary loader is %d of %d sectors, "
+		"First stage loader at 07C0:0000,\n"
+		" and the kernel command line is passed at "
+#ifndef LCF_LARGE_EBDA
+		"EBDA - 0x200\n",
+#else
+		"9000:%04X\n",
+#endif
+		(MAX_SECONDARY_FILESIZE+511)/512, MAX_SECONDARY
+#ifdef LCF_LARGE_EBDA
+		, ((FIRSTSEG-0x9000)<<4) + 0x200
+#endif
+								);        
+#endif
         printf("\n");
         if (version) return 0;
     }
-#if 0
-    if (((install || test || boot_device || disktab_file || compact) && !argc)
-      || (compact && linear && 0)) usage(name);
-#endif
-    if (!nowarn && compact && (linear || lba32))
-	fprintf(stderr,"Warning: COMPACT may conflict with %s on some "
-	  "systems\n", lba32 ? "LBA32" : "LINEAR");
     preload_types();
     fd = cfg_open(config_file);
     more = cfg_parse(cf_options);
+    if (tell_param) probe_tell(tell_param);
     if (!nowarn) {
 	if (fstat(fd,&st) < 0) {
 	    fprintf(stderr,"fstat %s: %s\n",config_file,strerror(errno));
@@ -459,7 +484,8 @@ int main(int argc,char **argv)
     }
     preload_dev_cache();
     if (identify) identify_image(identify,ident_opt);
-    if (strncmp("/dev/md",cfg_get_strg(cf_options,"boot"),7) == 0) {
+    if (cfg_get_strg(cf_options,"boot") != NULL &&
+        strncmp("/dev/md",cfg_get_strg(cf_options,"boot"),7) == 0) {
 	if ((md_fd=open(cfg_get_strg(cf_options,"boot"),O_NOACCESS)) < 0)
 	    die("Unable to open %s",cfg_get_strg(cf_options,"boot"));
 	if (fstat(md_fd,&st) < 0)
@@ -522,8 +548,8 @@ int main(int argc,char **argv)
 	    disktab = disk;
 	    if (cfg_get_strg(cf_options,"boot")) cfg_unset(cf_options,"boot");
 	    if (cfg_get_strg(cf_options,"map")) cfg_unset(cf_options,"map");
-	    strncpy(new_name,dev.name,8);
-	    new_name[8] = '\0';
+	    strncpy(new_name,dev.name,MAX_TOKEN);
+	    new_name[MAX_TOKEN] = '\0';
 	    cfg_set(cf_options,"boot",new_name,NULL);
 	    snprintf(new_name,MAX_TOKEN,"%s.%04x",md_boot_map,device);
 	    cfg_set(cf_options,"map",new_name,NULL);
@@ -543,6 +569,9 @@ int main(int argc,char **argv)
 	linear = cfg_get_flag(cf_options,"linear");
         lba32  = cfg_get_flag(cf_options,"lba32");
 	nowarn = cfg_get_flag(cf_options,"nowarn");
+	if (!nowarn && compact && (linear || lba32))
+	    fprintf(stderr,"Warning: COMPACT may conflict with %s on some "
+		"systems\n", lba32 ? "LBA32" : "LINEAR");
 	if (cfg_get_strg(cf_options,"verbose"))
 	    verbose += to_number(cfg_get_strg(cf_options,"verbose"));
 	if (reboot_arg) {
