@@ -183,7 +183,7 @@ static void map_compact(int dont_compact)
 	adj = ((walk->addr.device ^ walk->next->addr.device) & ~LBA32_NOCOUNT) == 0;
         if (adj && (walk->addr.device & LBA32_FLAG)) {
 	    if ((walk->addr.device & LBA32_NOCOUNT)==0) {
-	        if (adj = hinib==walk->next->addr.num_sect) {
+	        if ( (adj = (hinib==walk->next->addr.num_sect)) ) {
 		    walk->next->addr.num_sect = 1;
 		    walk->next->addr.device &= ~LBA32_NOCOUNT;
 		}
@@ -232,6 +232,7 @@ static void map_alloc_page(int offset,SECTOR_ADDR *addr)
 
     if ((here = lseek(map_file,offset,1)) < 0) pdie("lseek map file");
     if (write(map_file,"",1) != 1) pdie("write map file");
+    if (fsync(map_file)) pdie("fsync map file");
     if (!geo_comp_addr(&map_geo,here,addr))
 	die("Hole found in map file (alloc_page)");
     if (lseek(map_file,-offset-1,1) < 0) pdie("lseek map file");
